@@ -36,7 +36,6 @@ use bytes::Bytes;
 use chrono::{DateTime, Utc};
 use futures::stream::BoxStream;
 use futures::TryStreamExt;
-use reqwest::Client;
 use snafu::{OptionExt, ResultExt, Snafu};
 use std::collections::BTreeSet;
 use std::ops::Range;
@@ -51,8 +50,8 @@ use crate::aws::credential::{
 };
 use crate::multipart::{CloudMultiPartUpload, CloudMultiPartUploadImpl, UploadPart};
 use crate::{
-    GetResult, ListResult, MultipartId, ObjectMeta, ObjectStore, Path, Result,
-    RetryConfig, StreamExt,
+    ClientOptions, GetResult, ListResult, MultipartId, ObjectMeta, ObjectStore, Path,
+    Result, RetryConfig, StreamExt,
 };
 
 mod client;
@@ -109,6 +108,9 @@ enum Error {
     #[snafu(display("Missing SecretAccessKey"))]
     MissingSecretAccessKey,
 
+    #[snafu(display("Profile support requires aws_profile feature"))]
+    MissingProfileFeature,
+
     #[snafu(display("ETag Header missing from response"))]
     MissingEtag,
 
@@ -355,10 +357,11 @@ pub struct AmazonS3Builder {
     endpoint: Option<String>,
     token: Option<String>,
     retry_config: RetryConfig,
-    allow_http: bool,
     imdsv1_fallback: bool,
     virtual_hosted_style_request: bool,
     metadata_endpoint: Option<String>,
+    profile: Option<String>,
+    client_options: ClientOptions,
 }
 
 impl AmazonS3Builder {
@@ -370,13 +373,14 @@ impl AmazonS3Builder {
     /// Fill the [`AmazonS3Builder`] with regular AWS environment variables
     ///
     /// Variables extracted from environment:
-    /// * AWS_ACCESS_KEY_ID -> access_key_id
-    /// * AWS_SECRET_ACCESS_KEY -> secret_access_key
-    /// * AWS_DEFAULT_REGION -> region
-    /// * AWS_ENDPOINT -> endpoint
-    /// * AWS_SESSION_TOKEN -> token
-    /// * AWS_CONTAINER_CREDENTIALS_RELATIVE_URI -> <https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-iam-roles.html>
-    /// * AWS_ALLOW_HTTP -> set to "true" to permit HTTP connections without TLS
+    /// * `AWS_ACCESS_KEY_ID` -> access_key_id
+    /// * `AWS_SECRET_ACCESS_KEY` -> secret_access_key
+    /// * `AWS_DEFAULT_REGION` -> region
+    /// * `AWS_ENDPOINT` -> endpoint
+    /// * `AWS_SESSION_TOKEN` -> token
+    /// * `AWS_CONTAINER_CREDENTIALS_RELATIVE_URI` -> <https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-iam-roles.html>
+    /// * `AWS_ALLOW_HTTP` -> set to "true" to permit HTTP connections without TLS
+    /// * `AWS_PROFILE` -> set profile name, requires `aws_profile` feature enabled
     /// # Example
     /// ```
     /// use object_store::aws::AmazonS3Builder;
@@ -408,6 +412,10 @@ impl AmazonS3Builder {
             builder.token = Some(token);
         }
 
+        if let Ok(profile) = std::env::var("AWS_PROFILE") {
+            builder.profile = Some(profile);
+        }
+
         // This env var is set in ECS
         // https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-iam-roles.html
         if let Ok(metadata_relative_uri) =
@@ -418,7 +426,8 @@ impl AmazonS3Builder {
         }
 
         if let Ok(text) = std::env::var("AWS_ALLOW_HTTP") {
-            builder.allow_http = text == "true";
+            builder.client_options =
+                builder.client_options.with_allow_http(text == "true");
         }
 
         builder
@@ -474,7 +483,7 @@ impl AmazonS3Builder {
     /// * false (default):  Only HTTPS are allowed
     /// * true:  HTTP and HTTPS are allowed
     pub fn with_allow_http(mut self, allow_http: bool) -> Self {
-        self.allow_http = allow_http;
+        self.client_options = self.client_options.with_allow_http(allow_http);
         self
     }
 
@@ -528,6 +537,36 @@ impl AmazonS3Builder {
         self
     }
 
+    /// Set the proxy_url to be used by the underlying client
+    pub fn with_proxy_url(mut self, proxy_url: impl Into<String>) -> Self {
+        self.client_options = self.client_options.with_proxy_url(proxy_url);
+        self
+    }
+
+    /// Sets the client options, overriding any already set
+    pub fn with_client_options(mut self, options: ClientOptions) -> Self {
+        self.client_options = options;
+        self
+    }
+
+    /// Set the AWS profile name, see <https://docs.aws.amazon.com/cli/latest/userguide/cli-configure-profiles.html>
+    ///
+    /// This makes use of [aws-config] to provide credentials and therefore requires
+    /// the `aws-profile` feature to be enabled
+    ///
+    /// It is strongly encouraged that users instead make use of a credential manager
+    /// such as [aws-vault] not only to avoid the significant additional dependencies,
+    /// but also to avoid storing credentials in [plain text on disk]
+    ///
+    /// [aws-config]: https://docs.rs/aws-config
+    /// [aws-vault]: https://github.com/99designs/aws-vault
+    /// [plain text on disk]: https://99designs.com.au/blog/engineering/aws-vault/
+    #[cfg(feature = "aws_profile")]
+    pub fn with_profile(mut self, profile: impl Into<String>) -> Self {
+        self.profile = Some(profile.into());
+        self
+    }
+
     /// Create a [`AmazonS3`] instance from the provided values,
     /// consuming `self`.
     pub fn build(self) -> Result<AmazonS3> {
@@ -537,13 +576,13 @@ impl AmazonS3Builder {
         let credentials = match (self.access_key_id, self.secret_access_key, self.token) {
             (Some(key_id), Some(secret_key), token) => {
                 info!("Using Static credential provider");
-                CredentialProvider::Static(StaticCredentialProvider {
+                Box::new(StaticCredentialProvider {
                     credential: Arc::new(AwsCredential {
                         key_id,
                         secret_key,
                         token,
                     }),
-                })
+                }) as _
             }
             (None, Some(_), _) => return Err(Error::MissingAccessKeyId.into()),
             (Some(_), None, _) => return Err(Error::MissingSecretAccessKey.into()),
@@ -563,9 +602,13 @@ impl AmazonS3Builder {
                     let endpoint = format!("https://sts.{}.amazonaws.com", region);
 
                     // Disallow non-HTTPs requests
-                    let client = Client::builder().https_only(true).build().unwrap();
+                    let client = self
+                        .client_options
+                        .clone()
+                        .with_allow_http(false)
+                        .client()?;
 
-                    CredentialProvider::WebIdentity(WebIdentityProvider {
+                    Box::new(WebIdentityProvider {
                         cache: Default::default(),
                         token,
                         session_name,
@@ -573,24 +616,31 @@ impl AmazonS3Builder {
                         endpoint,
                         client,
                         retry_config: self.retry_config.clone(),
-                    })
+                    }) as _
                 }
-                _ => {
-                    info!("Using Instance credential provider");
+                _ => match self.profile {
+                    Some(profile) => {
+                        info!("Using profile \"{}\" credential provider", profile);
+                        profile_credentials(profile, region.clone())?
+                    }
+                    None => {
+                        info!("Using Instance credential provider");
 
-                    // The instance metadata endpoint is access over HTTP
-                    let client = Client::builder().https_only(false).build().unwrap();
+                        // The instance metadata endpoint is access over HTTP
+                        let client_options =
+                            self.client_options.clone().with_allow_http(true);
 
-                    CredentialProvider::Instance(InstanceCredentialProvider {
-                        cache: Default::default(),
-                        client,
-                        retry_config: self.retry_config.clone(),
-                        imdsv1_fallback: self.imdsv1_fallback,
-                        metadata_endpoint: self
-                            .metadata_endpoint
-                            .unwrap_or_else(|| METADATA_ENDPOINT.into()),
-                    })
-                }
+                        Box::new(InstanceCredentialProvider {
+                            cache: Default::default(),
+                            client: client_options.client()?,
+                            retry_config: self.retry_config.clone(),
+                            imdsv1_fallback: self.imdsv1_fallback,
+                            metadata_endpoint: self
+                                .metadata_endpoint
+                                .unwrap_or_else(|| METADATA_ENDPOINT.into()),
+                        }) as _
+                    }
+                },
             },
         };
 
@@ -619,15 +669,31 @@ impl AmazonS3Builder {
             bucket_endpoint,
             credentials,
             retry_config: self.retry_config,
-            allow_http: self.allow_http,
+            client_options: self.client_options,
         };
 
-        let client = Arc::new(S3Client::new(config));
+        let client = Arc::new(S3Client::new(config)?);
 
         Ok(AmazonS3 { client })
     }
 }
 
+#[cfg(feature = "aws_profile")]
+fn profile_credentials(
+    profile: String,
+    region: String,
+) -> Result<Box<dyn CredentialProvider>> {
+    Ok(Box::new(credential::ProfileProvider::new(profile, region)))
+}
+
+#[cfg(not(feature = "aws_profile"))]
+fn profile_credentials(
+    _profile: String,
+    _region: String,
+) -> Result<Box<dyn CredentialProvider>> {
+    Err(Error::MissingProfileFeature.into())
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -849,4 +915,34 @@ mod tests {
         let err = integration.delete(&location).await.unwrap_err();
         assert!(matches!(err, crate::Error::NotFound { .. }), "{}", err);
     }
+
+    #[tokio::test]
+    async fn s3_test_proxy_url() {
+        let s3 = AmazonS3Builder::new()
+            .with_access_key_id("access_key_id")
+            .with_secret_access_key("secret_access_key")
+            .with_region("region")
+            .with_bucket_name("bucket_name")
+            .with_allow_http(true)
+            .with_proxy_url("https://example.com")
+            .build();
+
+        assert!(s3.is_ok());
+
+        let err = AmazonS3Builder::new()
+            .with_access_key_id("access_key_id")
+            .with_secret_access_key("secret_access_key")
+            .with_region("region")
+            .with_bucket_name("bucket_name")
+            .with_allow_http(true)
+            .with_proxy_url("asdf://example.com")
+            .build()
+            .unwrap_err()
+            .to_string();
+
+        assert_eq!(
+            "Generic HTTP client error: builder error: unknown proxy scheme",
+            err
+        );
+    }
 }
