@@ -21,7 +21,10 @@ use crate::client::pagination::stream_paginated;
 use crate::client::retry::RetryExt;
 use crate::path::DELIMITER;
 use crate::util::{format_http_range, format_prefix};
-use crate::{BoxStream, ListResult, ObjectMeta, Path, Result, RetryConfig, StreamExt};
+use crate::{
+    BoxStream, ClientOptions, ListResult, ObjectMeta, Path, Result, RetryConfig,
+    StreamExt,
+};
 use bytes::{Buf, Bytes};
 use chrono::{DateTime, TimeZone, Utc};
 use itertools::Itertools;
@@ -121,9 +124,9 @@ pub struct AzureConfig {
     pub container: String,
     pub credentials: CredentialProvider,
     pub retry_config: RetryConfig,
-    pub allow_http: bool,
     pub service: Url,
     pub is_emulator: bool,
+    pub client_options: ClientOptions,
 }
 
 impl AzureConfig {
@@ -148,13 +151,9 @@ pub(crate) struct AzureClient {
 
 impl AzureClient {
     /// create a new instance of [AzureClient]
-    pub fn new(config: AzureConfig) -> Self {
-        let client = reqwest::ClientBuilder::new()
-            .https_only(!config.allow_http)
-            .build()
-            .unwrap();
-
-        Self { config, client }
+    pub fn new(config: AzureConfig) -> Result<Self> {
+        let client = config.client_options.client()?;
+        Ok(Self { config, client })
     }
 
     /// Returns the config
